@@ -107,8 +107,7 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::Query_execute(const Glib::ustring&
     {
       std::cout << "Debug: query string could not be converted to std::cout: " << ex.what() << std::endl;
     }
-    */
-
+   */
 
     result = gda_connection->execute_single_command(strQuery);
     if(!result)
@@ -734,103 +733,130 @@ Privileges Base_DB::get_table_privileges(const Glib::ustring& group_name, const
   return result;
 }
 
-void Base_DB::add_standard_tables() const
+bool Base_DB::add_standard_tables() const
 {
-  //Name, address, etc:
-  if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_PREFS_TABLE_NAME))
+  try
   {
-    TableInfo prefs_table_info;
-    prefs_table_info.m_name = GLOM_STANDARD_TABLE_PREFS_TABLE_NAME;
-    prefs_table_info.m_title = _("System: Preferences");
-    prefs_table_info.m_hidden = true;
-
-    Document_Glom::type_vecFields pref_fields;
-
-    Field primary_key; //It's not used, because there's only one record, but we must have one.
-    primary_key.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ID);
-    primary_key.set_glom_type(Field::TYPE_NUMERIC);
-    pref_fields.push_back(primary_key);
-
-    Field field_name;
-    field_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_NAME);
-    field_name.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_name);
-
-    Field field_org_name;
-    field_org_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME);
-    field_org_name.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_org_name);
-
-    Field field_org_address_street;
-    field_org_address_street.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET);
-    field_org_address_street.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_org_address_street);
-
-    Field field_org_address_street2;
-    field_org_address_street2.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2);
-    field_org_address_street2.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_org_address_street2);
-
-    Field field_org_address_town;
-    field_org_address_town.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN);
-    field_org_address_town.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_org_address_town);
-
-    Field field_org_address_county;
-    field_org_address_county.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY);
-    field_org_address_county.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_org_address_county);
-
-    Field field_org_address_country;
-    field_org_address_country.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY);
-    field_org_address_country.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_org_address_country);
-
-    Field field_org_address_postcode;
-    field_org_address_postcode.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE);
-    field_org_address_postcode.set_glom_type(Field::TYPE_TEXT);
-    pref_fields.push_back(field_org_address_postcode);
-
-    bool test = create_table(prefs_table_info, pref_fields);
-
-    if(test)
+    //Name, address, etc:
+    if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_PREFS_TABLE_NAME))
     {
-      //Add the single record:
-      Query_execute("INSERT INTO " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "(" GLOM_STANDARD_TABLE_PREFS_FIELD_ID ") VALUES (1)");
+      TableInfo prefs_table_info;
+      prefs_table_info.m_name = GLOM_STANDARD_TABLE_PREFS_TABLE_NAME;
+      prefs_table_info.m_title = _("System: Preferences");
+      prefs_table_info.m_hidden = true;
+
+      Document_Glom::type_vecFields pref_fields;
+
+      Field primary_key; //It's not used, because there's only one record, but we must have one.
+      primary_key.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ID);
+      primary_key.set_glom_type(Field::TYPE_NUMERIC);
+      pref_fields.push_back(primary_key);
+
+      Field field_name;
+      field_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_NAME);
+      field_name.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_name);
+
+      Field field_org_name;
+      field_org_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME);
+      field_org_name.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_org_name);
+
+      Field field_org_address_street;
+      field_org_address_street.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET);
+      field_org_address_street.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_org_address_street);
+
+      Field field_org_address_street2;
+      field_org_address_street2.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2);
+      field_org_address_street2.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_org_address_street2);
+
+      Field field_org_address_town;
+      field_org_address_town.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN);
+      field_org_address_town.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_org_address_town);
+
+      Field field_org_address_county;
+      field_org_address_county.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY);
+      field_org_address_county.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_org_address_county);
+
+      Field field_org_address_country;
+      field_org_address_country.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY);
+      field_org_address_country.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_org_address_country);
+
+      Field field_org_address_postcode;
+      field_org_address_postcode.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE);
+      field_org_address_postcode.set_glom_type(Field::TYPE_TEXT);
+      pref_fields.push_back(field_org_address_postcode);
+
+      bool test = create_table(prefs_table_info, pref_fields);
+
+      if(test)
+      {
+        //Add the single record:
+        Query_execute("INSERT INTO " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "(" GLOM_STANDARD_TABLE_PREFS_FIELD_ID ") VALUES (1)");
+
+        //Use the database title from the document, if there is one:
+        const Glib::ustring system_name = get_document()->get_database_title();
+        if(!system_name.empty())
+        {
+          Query_execute("UPDATE " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME " SET  " GLOM_STANDARD_TABLE_PREFS_FIELD_NAME " = '" + system_name + "' WHERE " GLOM_STANDARD_TABLE_PREFS_FIELD_ID " = 1");
+        }
+      }
+      else
+      {
+        g_warning("Base_DB::add_standard_tables(): create_table(prefs) failed.");
+        return false;
+      }
     }
+
+    //Auto-increment next values:
+    if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME))
+    {
+      TableInfo table_info;
+      table_info.m_name = GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME;
+      table_info.m_title = _("System: Auto Increments");
+      table_info.m_hidden = true;
+
+      Document_Glom::type_vecFields fields;
+
+      Field primary_key; //It's not used, because there's only one record, but we must have one.
+      primary_key.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_ID);
+      primary_key.set_glom_type(Field::TYPE_NUMERIC);
+      fields.push_back(primary_key);
+
+      Field field_table_name;
+      field_table_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME);
+      field_table_name.set_glom_type(Field::TYPE_TEXT);
+      fields.push_back(field_table_name);
+
+      Field field_field_name;
+      field_field_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME);
+      field_field_name.set_glom_type(Field::TYPE_TEXT);
+      fields.push_back(field_field_name);
+
+      Field field_next_value;
+      field_next_value.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE);
+      field_next_value.set_glom_type(Field::TYPE_TEXT);
+      fields.push_back(field_next_value);
+
+      const bool test = create_table(table_info, fields);
+      if(!test)
+      {
+        g_warning("Base_DB::add_standard_tables(): create_table(autoincrements) failed.");
+        return false;
+      }
+    }
+    else
+      return false;
   }
-
-  //Auto-increment next values:
-  if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME))
+  catch(const std::exception& ex)
   {
-    TableInfo table_info;
-    table_info.m_name = GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME;
-    table_info.m_title = _("System: Auto Increments");
-    table_info.m_hidden = true;
-
-    Document_Glom::type_vecFields fields;
-
-    Field primary_key; //It's not used, because there's only one record, but we must have one.
-    primary_key.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_ID);
-    primary_key.set_glom_type(Field::TYPE_NUMERIC);
-    fields.push_back(primary_key);
-
-    Field field_table_name;
-    field_table_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME);
-    field_table_name.set_glom_type(Field::TYPE_TEXT);
-    fields.push_back(field_table_name);
-
-    Field field_field_name;
-    field_field_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME);
-    field_field_name.set_glom_type(Field::TYPE_TEXT);
-    fields.push_back(field_field_name);
-
-    Field field_next_value;
-    field_next_value.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE);
-    field_next_value.set_glom_type(Field::TYPE_TEXT);
-    fields.push_back(field_next_value);
-
-    create_table(table_info, fields);
+    std::cerr << "Base_DB::add_standard_tables(): caught exception: " << ex.what() << std::endl;
+    return false;
   }
 }
 
@@ -865,7 +891,7 @@ void Base_DB::add_standard_groups()
   }
 }
 
-Gnome::Gda::Value Base_DB::auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name)
+Gnome::Gda::Value Base_DB::auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name) const
 {
   Gnome::Gda::Value value;
 
@@ -902,11 +928,45 @@ Gnome::Gda::Value Base_DB::auto_increment_insert_first_if_necessary(const Glib::
   return value;
 }
 
-Gnome::Gda::Value Base_DB::get_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name)
+void Base_DB::recalculate_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name) const
+{
+  //Make sure that the row exists in the glom system table:
+  auto_increment_insert_first_if_necessary(table_name, field_name);
+
+  //Get the max key value in the database:
+  const Glib::ustring sql_query = "SELECT MAX(" + table_name + "." + field_name + ") FROM " + table_name;
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+  if(datamodel && datamodel->get_n_rows() && datamodel->get_n_columns())
+  {
+    //Increment it:
+    const Gnome::Gda::Value value_max = datamodel->get_value_at(0, 0);
+    long num_max = util_decimal_from_string(value_max.to_string()); //TODO: Is this sensible? Probably not.
+    ++num_max;
+
+    //Set it in the glom system table:
+    const Gnome::Gda::Value next_value = GlomConversions::parse_value(num_max);
+    const Glib::ustring sql_query = "UPDATE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME " SET "
+      GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE " = " + next_value.to_string() + //TODO: Don't use to_string().
+      " WHERE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME " = '" + table_name + "' AND "
+                GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME " = '" + field_name + "'";
+
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    if(!datamodel)
+    {
+      g_warning("Base_DB::recalculate_next_auto_increment_value(): UPDATE failed.");
+    }
+  }
+  else
+  {
+    g_warning("Base_DB::recalculate_next_auto_increment_value(): SELECT MAX() failed.");
+  }
+}
+
+Gnome::Gda::Value Base_DB::get_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name) const
 {
   const Gnome::Gda::Value result = auto_increment_insert_first_if_necessary(table_name, field_name);
   long num_result = 0;
-  num_result = util_decimal_from_string(result.to_string());
+  num_result = util_decimal_from_string(result.to_string()); //TODO: Is this sensible? Probably not.
 
 
   //Increment the next_value:
@@ -1139,7 +1199,7 @@ void Base_DB::set_database_preferences(const SystemPrefs& prefs)
     {
       std::cerr << "Base_DB::set_database_preferences(): exception: " << ex.what() << std::endl;
     }
-    
+
     //Set some information in the document too, so we can use it to recreate the database:
     get_document()->set_database_title(prefs.m_name);
 }
@@ -1195,6 +1255,59 @@ bool Base_DB::create_table(const TableInfo& table_info, const Document_Glom::typ
   return table_creation_succeeded;
 }
 
+bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
+{
+  const Glib::ustring example_rows = get_document()->get_table_example_data(table_name);
+  if(example_rows.empty())
+    return true;
+
+  bool insert_succeeded = false;
+
+
+  //Get field names:
+  Document_Glom::type_vecFields vec_fields = get_document()->get_table_fields(table_name);
+  Glib::ustring strNames;
+  for(Document_Glom::type_vecFields::const_iterator iter = vec_fields.begin(); iter != vec_fields.end(); ++iter)
+  {
+    //Append it:
+    if(!strNames.empty())
+      strNames += ", ";
+
+    strNames += iter->get_name();
+  }
+
+  if(strNames.empty())
+  {
+    g_warning("Base_DB::insert_example_data(): strNames is empty.");
+  }
+
+  //Actually insert the data:
+  type_vecStrings vec_rows = string_separate(example_rows, "\n");
+
+  for(type_vecStrings::const_iterator iter = vec_rows.begin(); iter != vec_rows.end(); ++iter)
+  {
+    try
+    {
+      const Glib::ustring strQuery = "INSERT INTO " + table_name + " (" + strNames + ") VALUES (" + *iter + ")";
+      Query_execute(strQuery); //TODO: Test result.
+      insert_succeeded = true;
+    }
+    catch(const ExceptionConnection& ex)
+    {
+      insert_succeeded = false;
+      break;
+    }
+  }
+
+  for(Document_Glom::type_vecFields::const_iterator iter = vec_fields.begin(); iter != vec_fields.end(); ++iter)
+  {
+    if(iter->get_auto_increment())
+      recalculate_next_auto_increment_value(table_name, iter->get_name());
+  }
+
+  return insert_succeeded;
+}
+
 bool Base_DB::offer_field_list(LayoutItem_Field& field, const Glib::ustring& table_name)
 {
   bool result = false;
@@ -1548,7 +1661,297 @@ bool Base_DB::show_warning_no_records_found(Gtk::Window& transient_for)
   dialog.add_button(_("New Find"), Gtk::RESPONSE_OK);
   dialog.set_secondary_text(_("Your find criteria did not match any records in the table."));
   dialog.set_transient_for(transient_for);
-  
+
   const bool find_again = (dialog.run() == Gtk::RESPONSE_OK);
   return find_again;
 }
+
+Glib::ustring Base_DB::get_find_where_clause_quick(const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search) const
+{
+  Glib::ustring strClause;
+
+  const Document_Glom* document = get_document();
+  if(document)
+  {
+    //TODO: Cache the list of all fields, as well as caching (m_Fields) the list of all visible fields:
+    const Document_Glom::type_vecFields fields = document->get_table_fields(table_name);
+
+    type_vecLayoutFields fieldsToGet;
+    for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+    {
+      Glib::ustring strClausePart;
+
+      const Field& field = *iter;
+
+      bool use_this_field = true;
+      if(field.get_glom_type() != Field::TYPE_TEXT)
+      {
+          use_this_field = false;
+      }
+
+      if(use_this_field)
+      {
+        strClausePart = table_name + "." + field.get_name() + " " + field.sql_find_operator() + " " +  field.sql_find(quick_search);
+      }
+
+      if(!strClausePart.empty())
+      {
+        if(!strClause.empty())
+          strClause += " OR ";
+
+        strClause += strClausePart;
+      }
+    }
+  }
+
+  return strClause;
+}
+
+bool Base_DB::get_fields_for_table_one_field(const Glib::ustring& table_name, const Glib::ustring& field_name, Field& field) const
+{
+  //Initialize output parameter:
+  Field result = Field();
+
+  type_vecFields fields = get_fields_for_table(table_name);
+  type_vecFields::iterator iter = std::find_if(fields.begin(), fields.end(), predicate_FieldHasName<Field>(field_name));
+  if(iter != fields.end()) //TODO: Handle error?
+  {
+    field = *iter;
+    return true;
+  }
+
+  return false;
+}
+
+//static:
+bool Base_DB::get_field_primary_key_index_for_fields(const type_vecFields& fields, guint& field_column)
+{
+  //Initialize input parameter:
+  field_column = 0;
+
+  //TODO_performance: Cache the primary key?
+  guint col = 0;
+  guint cols_count = fields.size();
+  while(col < cols_count)
+  {
+    if(fields[col].get_primary_key())
+    {
+      field_column = col;
+      return true;
+    }
+    else
+    {
+      ++col;
+    }
+  }
+
+  return false; //Not found.
+}
+
+//static:
+bool Base_DB::get_field_primary_key_index_for_fields(const type_vecLayoutFields& fields, guint& field_column)
+{
+  //Initialize input parameter:
+  field_column = 0;
+
+  //TODO_performance: Cache the primary key?
+  guint col = 0;
+  guint cols_count = fields.size();
+  while(col < cols_count)
+  {
+    if(fields[col]->m_field.get_primary_key())
+    {
+      field_column = col;
+      return true;
+    }
+    else
+    {
+      ++col;
+    }
+  }
+
+  return false; //Not found.
+}
+
+bool Base_DB::get_field_primary_key_for_table(const Glib::ustring& table_name, Field& field) const
+{
+  const Document_Glom* document = get_document();
+  if(document)
+  {
+    //TODO_Performance:
+    Document_Glom::type_vecFields fields = document->get_table_fields(table_name);
+    for(Document_Glom::type_vecFields::iterator iter = fields.begin(); iter != fields.end(); ++iter)
+    {
+      if(iter->get_primary_key())
+      {
+        field = *iter;
+        return true;
+      }
+    }
+  }
+
+  return false;
+}
+
+void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const LayoutGroup& group, Base_DB::type_vecLayoutFields& vecFields) const
+{
+  //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): table_name=%s, all_db_fields.size()=%d, group.name=%s", table_name.c_str(), all_db_fields.size(), group.get_name().c_str());
+
+  const Document_Glom* document = get_document();
+
+  LayoutGroup::type_map_const_items items = group.get_items();
+  for(LayoutGroup::type_map_const_items::const_iterator iterItems = items.begin(); iterItems != items.end(); ++iterItems)
+  {
+    const LayoutItem* item = iterItems->second;
+
+    const LayoutItem_Field* item_field = dynamic_cast<const LayoutItem_Field*>(item);
+    if(item_field)
+    {
+      //Get the field info:
+      const Glib::ustring field_name = item->get_name();
+
+      if(item_field->get_has_relationship_name()) //If it's a field in a related table.
+      {
+        //Get the full field information:
+        const Glib::ustring relationship_name = item_field->get_relationship_name();
+        Relationship relationship;
+        bool test = document->get_relationship(table_name, relationship_name, relationship);
+        if(test)
+        {
+          Field field;
+          //TODO_Performance: get_fields_for_table_one_field() is probably very inefficient
+          bool test = get_fields_for_table_one_field(relationship.get_to_table(), item->get_name(), field);
+          if(test)
+          {
+            LayoutItem_Field layout_item = *item_field; //TODO_Performance: Reduce the copying.
+            layout_item.m_field = field; //Fill in the full field information for later.
+
+            //TODO_Performance: We do this once for each related field, even if there are 2 from the same table:
+            const Privileges privs_related = get_current_privs(relationship.get_to_table());
+            layout_item.m_priv_view = privs_related.m_view;
+            layout_item.m_priv_edit = privs_related.m_edit;
+
+            vecFields.push_back( sharedptr<LayoutItem_Field>(new LayoutItem_Field(layout_item)) );
+          }
+        }
+      }
+      else //It's a regular field in the table:
+      {
+        type_vecFields::const_iterator iterFind = std::find_if(all_db_fields.begin(), all_db_fields.end(), predicate_FieldHasName<Field>(field_name));
+
+        //If the field does not exist anymore then we won't try to show it:
+        if(iterFind != all_db_fields.end() )
+        {
+          LayoutItem_Field layout_item = *item_field; //TODO_Performance: Reduce the copying here.
+          layout_item.m_field = *iterFind; //Fill the LayoutItem with the full field information.
+
+          //Prevent editing of the field if the user may not edit this table:
+          layout_item.m_priv_view = table_privs.m_view;
+          layout_item.m_priv_edit = table_privs.m_edit;
+
+          vecFields.push_back( sharedptr<LayoutItem_Field>(new LayoutItem_Field(layout_item)) );
+        }
+      }
+    }
+    else
+    {
+      const LayoutGroup* item_group = dynamic_cast<const LayoutGroup*>(item);
+      if(item_group)
+      {
+        //Recurse:
+        get_table_fields_to_show_for_sequence_add_group(table_name, table_privs, all_db_fields, *item_group, vecFields);
+      }
+    }
+  }
+
+  if(vecFields.empty())
+  {
+    //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): Returning empty list.");
+  }
+}
+
+Base_DB::type_vecLayoutFields Base_DB::get_table_fields_to_show_for_sequence(const Glib::ustring& table_name, const Document_Glom::type_mapLayoutGroupSequence& mapGroupSequence) const
+{
+  //Get field definitions from the database, with corrections from the document:
+  type_vecFields all_fields = get_fields_for_table(table_name);
+
+  const Privileges table_privs = get_current_privs(table_name);
+
+  //Get fields that the document says we should show:
+  type_vecLayoutFields result;
+  const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
+  if(pDoc)
+  {
+    if(mapGroupSequence.empty())
+    {
+      //No field sequence has been saved in the document, so we use all fields by default, so we start with something visible:
+
+      //Start with the Primary Key as the first field:
+      guint iPrimaryKey = 0;
+      bool bPrimaryKeyFound = get_field_primary_key_index_for_fields(all_fields, iPrimaryKey);
+      Glib::ustring primary_key_field_name;
+      if(bPrimaryKeyFound)
+      {
+        sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+        layout_item->m_field = all_fields[iPrimaryKey];
+
+        //Don't use thousands separators with ID numbers:
+        layout_item->m_formatting.m_numeric_format.m_use_thousands_separator = false;
+
+        layout_item->set_editable(true); //A sensible default.
+
+        //Prevent editing of the field if the user may not edit this table:
+        layout_item->m_priv_view = table_privs.m_view;
+        layout_item->m_priv_edit = table_privs.m_edit;
+
+        result.push_back(layout_item);
+      }
+
+      //Add the rest:
+      for(type_vecFields::const_iterator iter = all_fields.begin(); iter != all_fields.end(); ++iter)
+      {
+        const Field& field_info = *iter;
+
+        if(iter->get_name() != primary_key_field_name) //We already added the primary key.
+        {
+          sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+          layout_item->m_field = field_info;
+
+          layout_item->set_editable(true); //A sensible default.
+
+          //Prevent editing of the field if the user may not edit this table:
+          layout_item->m_priv_view = table_privs.m_view;
+          layout_item->m_priv_edit = table_privs.m_edit;
+
+          result.push_back(layout_item);
+        }
+      }
+    }
+    else
+    {
+      type_vecFields vecFieldsInDocument = pDoc->get_table_fields(table_name);
+
+      //We will show the fields that the document says we should:
+      for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroupSequence.begin(); iter != mapGroupSequence.end(); ++iter)
+      {
+        const LayoutGroup& group = iter->second;
+
+        if(true) //!group.m_hidden)
+        {
+          //Get the fields:
+          get_table_fields_to_show_for_sequence_add_group(table_name, table_privs, all_fields, group, result);
+        }
+      }
+    }
+  }
+
+  if(result.empty())
+  {
+    //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): Returning empty list.");
+  }
+
+  return result;
+}
+
+
+
