@@ -74,15 +74,16 @@ public:
   virtual Glib::RefPtr<Gnome::Gda::DataModel> Query_execute(const Glib::ustring& strQuery) const;
 
   void add_standard_groups();
-  void add_standard_tables() const;
+  bool add_standard_tables() const;
 
   bool create_table(const TableInfo& table_info, const Document_Glom::type_vecFields& fields) const;
+  bool insert_example_data(const Glib::ustring& table_name) const;
 
   typedef std::vector< sharedptr<LayoutItem_Field> > type_vecLayoutFields;
 
 protected:
   bool offer_field_list(LayoutItem_Field& field, const Glib::ustring& table_name);
-  
+
   ///@result Whether the user would like to find again.
   static bool show_warning_no_records_found(Gtk::Window& transient_for);
 
@@ -97,6 +98,11 @@ protected:
   bool get_table_exists_in_database(const Glib::ustring& table_name) const;
 
   type_vecFields get_fields_for_table(const Glib::ustring& table_name) const;
+  bool get_fields_for_table_one_field(const Glib::ustring& table_name, const Glib::ustring& field_name, Field& field) const;
+
+  bool get_field_primary_key_for_table(const Glib::ustring& table_name, Field& field) const;
+
+  Glib::ustring get_find_where_clause_quick(const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search) const;
 
   type_vecStrings get_database_groups() const;
   type_vecStrings get_database_users(const Glib::ustring& group_name = Glib::ustring()) const;
@@ -118,18 +124,30 @@ protected:
   ///@show_null_records means show a summary of zero values when there are no records to actually summarise.
   void report_build_records(const Glib::ustring& table_name, xmlpp::Element& parent_node, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& sort_clause = Glib::ustring(), bool one_record_only = false);
 
-  Gnome::Gda::Value auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name);
+  Gnome::Gda::Value auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
 
   /** Get the next auto-increment value for this primary key, from the glom system table.
    * Add a row for this field in the system table if it does not exist already.
    */
-  Gnome::Gda::Value get_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name);
+  Gnome::Gda::Value get_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
+
+  /** Set the next auto-increment value in the glom system table, by examining all current values.
+   * Use this, for instance, after importing rows.
+   * Add a row for this field in the system table if it does not exist already.
+   */
+  void recalculate_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
 
   virtual bool fill_from_database();
   virtual void fill_end(); //Call this from the end of fill_from_database() overrides.
 
   virtual void on_userlevel_changed(AppState::userlevels userlevel);
 
+  type_vecLayoutFields get_table_fields_to_show_for_sequence(const Glib::ustring& table_name, const Document_Glom::type_mapLayoutGroupSequence& mapGroupSequence) const;
+  void get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const LayoutGroup& group, type_vecLayoutFields& vecFields) const;
+
+  static bool get_field_primary_key_index_for_fields(const type_vecFields& fields, guint& field_column);
+  static bool get_field_primary_key_index_for_fields(const type_vecLayoutFields& fields, guint& field_column);
+
   static Glib::ustring util_string_from_decimal(guint decimal);
   static guint util_decimal_from_string(const Glib::ustring& str);
 
