@@ -54,7 +54,15 @@ namespace GlomConversions
   Gnome::Gda::Value get_empty_value(Field::glom_field_type field_type);
 
   Gnome::Gda::Value get_example_value(Field::glom_field_type field_type);
+
+  Glib::ustring get_escaped_binary_data(guint8* buffer, size_t buffer_size);
+  Gnome::Gda::Value parse_escaped_binary_data(const Glib::ustring& escaped_data);
 }
 
+//Copied from Postgres's PQunescapeBytea() so I don't have the trouble of finding and linking to the 
+//postgres libraries directly, without the benefit of a pkg-config .pc file. murrayc.
+unsigned char *
+Glom_PQunescapeBytea(const unsigned char *strtext, size_t *retbuflen);
+
 #endif //GLOM_DATASTRUCTURE_GLOMCONVERSIONS_H
 
