@@ -29,6 +29,9 @@
 #include "data_structure/layout/report_parts/layoutitem_summary.h"
 #include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
 #include "relationships_overview/dialog_relationships_overview.h"
+#include "filechooser_export.h"
+#include <sstream> //For stringstream.
+#include <fstream>
 #include <glibmm/i18n.h>
 
 Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
@@ -37,6 +40,13 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_pLabel_Table(0),
   m_pLabel_Mode(0),
   m_pLabel_userlevel(0),
+  m_pBox_QuickFind(0),
+  m_pEntry_QuickFind(0),
+  m_pButton_QuickFind(0),
+  m_pBox_RecordsCount(0),
+  m_pLabel_RecordsCount(0),
+  m_pLabel_FoundCount(0),
+  m_pButton_FindAll(0),
   m_pBox_Mode(0),
   m_pBox_Tables(0),
   m_pBox_Reports(0),
@@ -53,6 +63,25 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   refGlade->get_widget("label_table_name", m_pLabel_Table);
   refGlade->get_widget("label_mode", m_pLabel_Mode);
   refGlade->get_widget("label_user_level", m_pLabel_userlevel);
+
+  refGlade->get_widget("hbox_quickfind", m_pBox_QuickFind);
+  m_pBox_QuickFind->hide();
+
+  refGlade->get_widget("entry_quickfind", m_pEntry_QuickFind);
+  m_pEntry_QuickFind->signal_activate().connect(
+   sigc::mem_fun(*this, &Frame_Glom::on_button_quickfind) ); //Pressing Enter here is like pressing Find.
+
+  refGlade->get_widget("button_quickfind", m_pButton_QuickFind);
+  m_pButton_QuickFind->signal_clicked().connect(
+    sigc::mem_fun(*this, &Frame_Glom::on_button_quickfind) );
+
+  refGlade->get_widget("hbox_records_count", m_pBox_RecordsCount);
+  refGlade->get_widget("label_records_count", m_pLabel_RecordsCount);
+  refGlade->get_widget("label_records_found_count", m_pLabel_FoundCount);
+  refGlade->get_widget("button_find_all", m_pButton_FindAll);
+  m_pButton_FindAll->signal_clicked().connect(
+    sigc::mem_fun(*this, &Frame_Glom::on_button_find_all) );
+
   refGlade->get_widget_derived("vbox_mode", m_pBox_Mode);
 
   //m_pLabel_Mode->set_text(_("No database selected.\n Use the Navigation menu, or open a previous Glom document."));
@@ -77,7 +106,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
 
     refXml->get_widget_derived("box_navigation_tables", m_pBox_Tables);
     m_pDialog_Tables = new Dialog_Glom(m_pBox_Tables);
-    
+
     //Respond to window close:
     m_pDialog_Tables->signal_hide().connect(sigc::mem_fun(*this, &Frame_Glom::on_dialog_tables_hide));
   }
@@ -175,8 +204,11 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_pBox_Reports->signal_selected.connect(sigc::mem_fun(*this, &Frame_Glom::on_box_reports_selected));
 
   m_Notebook_Find.signal_find_criteria.connect(sigc::mem_fun(*this, &Frame_Glom::on_notebook_find_criteria));
+  m_Notebook_Find.show();
 
   m_Notebook_Data.signal_record_details_requested().connect(sigc::mem_fun(*this, &Frame_Glom::on_notebook_data_record_details_requested));
+  m_Notebook_Data.show();
+
   //Fill Composite View:
   //This means that set_document and load/save are delegated to these children:
   add_view(m_pBox_Tables);
@@ -188,8 +220,6 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   add_view(&m_Notebook_Find); //Also a composite view.
 
   on_userlevel_changed(AppState::USERLEVEL_OPERATOR); //A default to show before a document is created or loaded.
-
-  show_all();
 }
 
 Frame_Glom::~Frame_Glom()
@@ -252,6 +282,7 @@ void Frame_Glom::set_databases_selected(const Glib::ustring& strName)
 
   get_document()->set_connection_database(strName);
 
+  show_system_name();
   do_menu_Navigate_Table(true /* open default */);
 }
 
@@ -294,7 +325,8 @@ void Frame_Glom::set_mode_widget(Gtk::Widget& widget)
 
 bool Frame_Glom::set_mode(enumModes mode)
 {
-  bool bChanged = (m_Mode != mode);
+  //TODO: This seems to be called twice when changing mode.
+  const bool changed = (m_Mode != mode);
 
   //Choose a default mode, if necessary:
   if(mode == MODE_None)
@@ -303,7 +335,32 @@ bool Frame_Glom::set_mode(enumModes mode)
   m_Mode_Previous = m_Mode;
   m_Mode = mode;
 
-  return bChanged;
+  //Hide the Quick Find widgets if we are not in Find mode.
+  const bool show_quickfind = (m_Mode == MODE_Find);
+  if(show_quickfind)
+  {
+    m_pBox_QuickFind->show();
+
+    //Clear the quick-find entry, ready for a new Find.
+    if(changed)
+    {
+      m_pEntry_QuickFind->set_text(Glib::ustring());
+
+      //Put the cursor in the quick find entry:
+      m_pEntry_QuickFind->grab_focus();
+      //m_pButton_QuickFind->grab_default();
+    }
+
+    m_pBox_RecordsCount->hide();
+  }
+  else
+  {
+    m_pBox_QuickFind->hide();
+
+    m_pBox_RecordsCount->show();
+  }
+
+  return changed;
 }
 
 void Frame_Glom::alert_no_table()
@@ -314,7 +371,7 @@ void Frame_Glom::alert_no_table()
   {
     //TODO: Obviously this document should have been deleted when the database-creation was cancelled.
     /* Note that "canceled" is the correct US spelling. */
-    show_ok_dialog(_("No database"), _("This document does not specify any database. Maybe the document creation was canceled before the database could be created."), *pWindowApp);
+    show_ok_dialog(_("No table"), _("This database has no tables yet."), *pWindowApp, Gtk::MESSAGE_WARNING);
   }
 }
 
@@ -343,7 +400,7 @@ void Frame_Glom::show_table(const Glib::ustring& strTableName)
     //Update user-level dependent UI:
     if(pApp)
       on_userlevel_changed(pApp->get_userlevel());
- 
+
     switch(m_Mode)
     {
       case(MODE_Data):
@@ -351,6 +408,10 @@ void Frame_Glom::show_table(const Glib::ustring& strTableName)
         strMode = _("Data");
         m_Notebook_Data.init_db_details(m_strTableName);
         set_mode_widget(m_Notebook_Data);
+
+        //Show how many records were found:
+        update_records_count();
+
         break;
       }
       case(MODE_Find):
@@ -376,7 +437,7 @@ void Frame_Glom::show_table(const Glib::ustring& strTableName)
   //List the reports in the menu:
   pApp->fill_menu_reports(strTableName);
 
-  show_all();
+  //show_all();
 }
 
 void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action, const Glib::RefPtr<Gtk::RadioAction>& operator_action)
@@ -428,6 +489,154 @@ void Frame_Glom::on_menu_userlevel_Operator(const Glib::RefPtr<Gtk::RadioAction>
   }
 }
 
+
+void Frame_Glom::on_menu_file_export()
+{
+  //Start with a sequence based on the Details view:
+  //The user can changed this by clicking the button in the FileChooser:
+  Document_Glom::type_mapLayoutGroupSequence mapGroupSequence =  get_document()->get_data_layout_groups_plus_new_fields("details", m_strTableName);
+
+  Gtk::Window* pWindowApp = get_app_window();
+  g_assert(pWindowApp);
+
+  //Do not try to export the data if the user may not view it:
+  Privileges table_privs = get_current_privs(m_strTableName);
+  if(!table_privs.m_view)
+  {
+    show_ok_dialog(_("Export Not Allowed."), _("You do not have permission to view the data in this table, so you may not export the data."), *pWindowApp, Gtk::MESSAGE_ERROR);
+    return;
+  }
+
+  //Ask the user for the new file location, and to optionally modify the format:
+  FileChooser_Export dialog;
+  dialog.set_export_layout(mapGroupSequence, m_strTableName, get_document());
+  const int response = dialog.run();
+  dialog.hide();
+
+  if(response == Gtk::RESPONSE_CANCEL)
+    return;
+
+  const std::string filepath = dialog.get_filename();
+  if(filepath.empty())
+    return;
+
+  dialog.get_layout_groups(mapGroupSequence);
+
+  //const int index_primary_key = fieldsSequence.size() - 1;
+
+  const Glib::ustring found_set_where_clause = m_Notebook_Data.get_where_clause();
+
+  std::fstream the_stream(filepath.c_str(), std::ios_base::out | std::ios_base::trunc);
+  if(!the_stream)
+  {
+      show_ok_dialog(_("Could Not Create File."), _("Glom could not create the specified file."), *pWindowApp, Gtk::MESSAGE_ERROR);
+    return;
+  }
+
+  export_data_to_stream(the_stream, m_strTableName, mapGroupSequence, found_set_where_clause);
+}
+
+void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause)
+{
+  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(table_name, sequence);
+
+  if(fieldsSequence.empty())
+    return;
+
+  const Glib::ustring query = GlomUtils::build_sql_select_with_where_clause(table_name, fieldsSequence, where_clause);
+
+  //TODO: Lock the database (prevent changes) during export.
+  Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+
+  guint rows_count = 0;
+  if(result)
+    rows_count = result->get_n_rows();
+
+  if(rows_count)
+  {
+    const guint columns_count = result->get_n_columns();
+
+    for(guint row_index = 0; row_index < rows_count; ++row_index)
+    {
+        std::string row_string;
+
+        for(guint col_index = 0; col_index < columns_count; ++col_index)
+        {
+          const Gnome::Gda::Value value = result->get_value_at(col_index, row_index);
+
+          sharedptr<LayoutItem_Field> layout_item = fieldsSequence[col_index];
+          //if(layout_item->m_field.get_glom_type() != Field::TYPE_IMAGE) //This is too much data.
+          //{
+            if(!row_string.empty())
+              row_string += ",";
+
+            //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
+            row_string += layout_item->m_field.sql(value);
+
+            //if(layout_item->m_field.get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
+            //{
+             //std::cout << "  field name=" << layout_item->get_name() << ", value=" << layout_item->m_field.sql(value) << std::endl;
+            //}
+        }
+
+        //std::cout << " row_string=" << row_string << std::endl;
+        the_string += (row_string += "\n");
+    }
+  }
+}
+
+void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause)
+{
+  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(table_name, sequence);
+
+  if(fieldsSequence.empty())
+    return;
+
+  const Glib::ustring query = GlomUtils::build_sql_select_with_where_clause(table_name, fieldsSequence, where_clause);
+
+  //TODO: Lock the database (prevent changes) during export.
+  Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+
+  guint rows_count = 0;
+  if(result)
+    rows_count = result->get_n_rows();
+
+  if(rows_count)
+  {
+    const guint columns_count = result->get_n_columns();
+
+    for(guint row_index = 0; row_index < rows_count; ++row_index)
+    {
+        std::string row_string;
+
+        for(guint col_index = 0; col_index < columns_count; ++col_index)
+        {
+          const Gnome::Gda::Value value = result->get_value_at(col_index, row_index);
+
+          sharedptr<LayoutItem_Field> layout_item = fieldsSequence[col_index];
+          //if(layout_item->m_field.get_glom_type() != Field::TYPE_IMAGE) //This is too much data.
+          //{
+            if(!row_string.empty())
+              row_string += ",";
+
+            //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
+            row_string += layout_item->m_field.sql(value);
+
+            if(layout_item->m_field.get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
+            {
+              if(!GlomConversions::value_is_empty(value))
+                std::cout << "  field name=" << layout_item->get_name() << ", image value not empty=" << std::endl;
+            }
+            //std::cout << "  field name=" << layout_item->get_name() << ", value=" << layout_item->m_field.sql(value) << std::endl;
+          //}
+        }
+
+        //std::cout << " row_string=" << row_string << std::endl;
+        the_stream << row_string << std::endl;
+    }
+  }
+}
+
 void Frame_Glom::on_menu_file_print()
 {
  Notebook_Glom* notebook_current = dynamic_cast<Notebook_Glom*>(m_pBox_Mode->get_child());
@@ -543,16 +752,35 @@ Gtk::Window* Frame_Glom::get_app_window()
 
 }
 
-void Frame_Glom::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent)
+void Frame_Glom::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent, Gtk::MessageType message_type)
 {
-  Gtk::MessageDialog dialog("<b>" + title + "</b>", true /* markup */, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK);
+  Gtk::MessageDialog dialog("<b>" + title + "</b>", true /* markup */, message_type, Gtk::BUTTONS_OK);
   dialog.set_secondary_text(message);
   dialog.set_transient_for(parent);
   dialog.run();
 }
 
+void Frame_Glom::on_button_quickfind()
+{
+  const Glib::ustring criteria = m_pEntry_QuickFind->get_text();
+  if(criteria.empty())
+  {
+    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No Find Criteria")), true, Gtk::MESSAGE_WARNING );
+    dialog.set_secondary_text(_("You have not entered any quick find criteria."));
+    dialog.set_transient_for(*get_app_window());
+    dialog.run();
+  }
+  else
+  {
+    Glib::ustring where_clause = get_find_where_clause_quick(m_strTableName, Gnome::Gda::Value(criteria));
+    //std::cout << "Frame_Glom::on_button_quickfind(): where_clause=" << where_clause << std::endl;
+    on_notebook_find_criteria(where_clause);
+  }
+}
+
 void Frame_Glom::on_notebook_find_criteria(const Glib::ustring& strWhereClause)
 {
+  //std::cout << "Frame_Glom::on_notebook_find_criteria(): " << strWhereClause << std::endl;
   //on_menu_Mode_Data();
 
   App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
@@ -560,6 +788,7 @@ void Frame_Glom::on_notebook_find_criteria(const Glib::ustring& strWhereClause)
   {
     pApp->set_mode_data();
 
+    //std::cout << "Frame_Glom::on_notebook_find_criteria: where_clause=" << strWhereClause << std::endl;
     bool records_found = m_Notebook_Data.init_db_details(m_strTableName, strWhereClause);
     m_Notebook_Data.select_page_for_find_results();
 
@@ -570,6 +799,11 @@ void Frame_Glom::on_notebook_find_criteria(const Glib::ustring& strWhereClause)
       if(find_again)
         pApp->set_mode_find();
     }
+    else
+    {
+      //Show how many records were found:
+      update_records_count();
+    }
   }
 }
 
@@ -713,6 +947,7 @@ void Frame_Glom::show_system_name()
 
   m_pLabel_Name->set_text ( Bakery::App_Gtk::util_bold_message(system_name) );
   m_pLabel_Name->set_use_markup();
+  m_pLabel_Name->show();
 }
 
 
@@ -892,6 +1127,77 @@ void Frame_Glom::on_developer_dialog_hide()
   show_table(m_strTableName);
 }
 
+bool Frame_Glom::connection_request_password_and_choose_new_database_name()
+{
+  //Ask for connection details:
+  m_pDialogConnection->load_from_document(); //Get good defaults.
+  m_pDialogConnection->set_transient_for(*get_app_window());
+  int response = m_pDialogConnection->run();
+  m_pDialogConnection->hide();
+
+  if(response == Gtk::RESPONSE_OK)
+  {
+    const Glib::ustring database_name = get_document()->get_connection_database();
+    bool keep_trying = true;
+    size_t extra_num = 0;
+    while(keep_trying)
+    {
+      Glib::ustring database_name_possible;
+      if(extra_num == 0)
+        database_name_possible = database_name; //Try the original name first.
+      else
+      {
+        //Create a new database name by appending a number to the original name:
+        char pchExtraNum[10];
+        sprintf(pchExtraNum, "%d", extra_num);
+        database_name_possible = (database_name + pchExtraNum);
+      }
+      ++extra_num;
+
+      m_pDialogConnection->set_database_name(database_name_possible);
+      //std::cout << "possible name=" << database_name_possible << std::endl;
+
+      try
+      {
+        sharedptr<SharedConnection> sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
+        //If no exception was thrown then the database exists.
+        //But we are looking for an unused database name, so we will try again.
+      }
+      catch(const ExceptionConnection& ex)
+      {
+        //g_warning("Frame_Glom::connection_request_password_and_choose_new_database_name(): caught exception.");
+
+        if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
+        {
+          //Warn the user, and let him try again:
+          m_pDialogConnectionFailed->set_transient_for(*get_app_window());
+          int response = m_pDialogConnectionFailed->run();
+          m_pDialogConnectionFailed->hide();
+
+          //TODO: Combine these into one dialog.
+          if(response != Gtk::RESPONSE_OK)
+            return false; //The user cancelled.
+
+          response = m_pDialogConnection->run();
+          m_pDialogConnection->hide();
+          if(response != Gtk::RESPONSE_OK)
+            return false; //The user cancelled.
+        }
+        else
+        {
+          std::cout << "Frame_Glom::connection_request_password_and_choose_new_database_name(): unused database name successfully found: " << database_name_possible << std::endl; 
+          //The connection to the server is OK, but the specified database does not exist.
+          //That's good - we were looking for an unused database name.
+          get_document()->set_connection_database(database_name_possible);
+          return true;
+        }
+      }
+    }
+  }
+  else
+    return false; //The user cancelled.
+}
+
 bool Frame_Glom::connection_request_password_and_attempt()
 {
   while(true) //Loop until a return
@@ -941,7 +1247,7 @@ bool Frame_Glom::connection_request_password_and_attempt()
   }
 }
 
-bool Frame_Glom::create_database(const Glib::ustring& database_name, bool request_password)
+bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib::ustring& title, bool request_password)
 {
   //Ask for connection details:
   bool connection_possible = false;
@@ -950,17 +1256,20 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, bool reques
     if(request_password)
       connection_possible = connection_request_password_and_attempt(); //If it succeeded and the user did not cancel.
     else
-      connection_possible = true; //Assume that connection details are already coorect.
+    {
+      m_pDialogConnection->set_database_name(Glib::ustring()); //Make sure that it always connects to the default database when creating a database.
+      connection_possible = true; //Assume that connection details are already correct.
+    }
   }
   catch(const ExceptionConnection& ex)
   {
      connection_possible = false;
-     g_warning("debug Frame_Glom::create_database() exception caught: connection failed be");
+     std::cerr << "debug Frame_Glom::create_database() exception caught: connection failed: " << ex.what() << std::endl;
   }
 
   if(!connection_possible)
   {
-    g_warning("debug Frame_Glom::create_database(): connection was not possible.");
+    //g_warning("debug Frame_Glom::create_database(): connection was not possible.");
     return false;
   }
   else
@@ -972,11 +1281,15 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, bool reques
       if(request_password)
         sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
       else
-        sharedconnection = ConnectionPool::get_instance()->connect();
+      {
+        ConnectionPool* connection_pool = ConnectionPool::get_instance();
+        connection_pool->set_database(Glib::ustring()); //Make sure that it uses the default database for connections when creating databases.
+        sharedconnection = connection_pool->connect();
+      }
     }
     catch(const ExceptionConnection& ex)
     {
-      g_warning("debug Frame_Glom::create_database() Connection failed.");
+      //g_warning("debug Frame_Glom::create_database() Connection failed.");
 
       return false;
     }
@@ -991,22 +1304,57 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, bool reques
         bool result = connection->create_database(database_name);
         if(result)
         {
-          g_warning("Frame_Glom::create_database(): Succeeded: database_name=%s", database_name.c_str());
-          return true; // It succeeded.
-        }
-        else
-        {
-          handle_error();
-
-          //The calling function will tell the user.
-
-          return false; //Failed. //TODO: Allow the user to try with a different user name?
+          std::cout << "Frame_Glom::create_database(): Creation succeeded: database_name=" << database_name << std::endl;
         }
       }
     }
   }
 
-  return false;
+  //Connect to the actual database:
+  ConnectionPool* connection_pool = ConnectionPool::get_instance();
+  connection_pool->set_database(database_name);
+
+  sharedptr<SharedConnection> sharedconnection;
+  try
+  {
+    sharedconnection = connection_pool->connect();
+  }
+  catch(const std::exception& ex)
+  {
+    std::cerr << "Frame_Glom::create_database(): Could not connect to just-created database. exception caught:" << ex.what() << std::endl;
+    return false;
+  }
+
+  if(sharedconnection)
+  {
+    add_standard_tables(); //Add internal, hidden, tables.
+
+    //Create the developer group, and make this user a member of it:
+    //If we got this far then the user must really have developer privileges already:
+    add_standard_groups();
+
+    std::cout << "Frame_Glom::create_database(): Creation of standard tables and groups finished." << std::endl;
+
+    //Set the title based on the title in the example document, or the user-supplied title when creating new documents:
+    SystemPrefs prefs = get_database_preferences();
+    if(prefs.m_name.empty())
+    {
+      std::cout << "Frame_Glom::create_database(): Setting title in the database." << std::endl;
+      prefs.m_name = title;
+      set_database_preferences(prefs);
+    }
+    else
+    {
+      std::cout << "Frame_Glom::create_database(): database has title: " << prefs.m_name << std::endl;
+    }
+
+    return true;
+  }
+  else
+  {
+    std::cerr << "Frame_Glom::create_database(): Could not connect to just-created database." << std::endl;
+    return false;
+  }
 }
 
 
@@ -1068,7 +1416,7 @@ void Frame_Glom::on_dialog_tables_hide()
       App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
       if(pApp)
         pApp->fill_menu_tables();
-        
+
       //Select a different table if the current one no longer exists:
       if(!document->get_table_is_known(m_strTableName))
       {
@@ -1076,7 +1424,7 @@ void Frame_Glom::on_dialog_tables_hide()
         Glib::ustring table_name = document->get_default_table();
         if(table_name.empty())
           table_name = document->get_first_table();
-    
+
         show_table(table_name);
       }
     }
@@ -1089,3 +1437,48 @@ void Frame_Glom::on_notebook_data_record_details_requested(const Glib::ustring&
   m_Notebook_Data.show_details(primary_key_value);
 }
 
+void Frame_Glom::update_records_count()
+{
+  //Get the number of records available and the number found,
+  //and all the user to find all if necessary.
+
+  gulong count_all = 0;
+  gulong count_found = 0;
+  m_Notebook_Data.get_record_counts(count_all, count_found);
+
+  std::string str_count_all, str_count_found;
+
+  std::stringstream the_stream;
+  //the_stream.imbue( current_locale );
+  the_stream << count_all;
+  the_stream >> str_count_all;
+
+  if(count_found == count_all)
+  {
+    if(count_found != 0) //Show 0 instead of "all" when all of no records are found, to avoid confusion.
+      str_count_found = _("All");
+    else
+      str_count_found = str_count_all;
+
+    m_pButton_FindAll->hide();
+  }
+  else
+  {
+    std::stringstream the_stream; //Reusing the existing stream seems to produce an empty string.
+    the_stream << count_found;
+    the_stream >> str_count_found;
+
+    m_pButton_FindAll->show();
+  }
+
+  m_pLabel_RecordsCount->set_text(str_count_all);
+  m_pLabel_FoundCount->set_text(str_count_found);
+
+}
+
+void Frame_Glom::on_button_find_all()
+{
+  //Change the found set to all records:
+  show_table(m_strTableName);
+}
+
