@@ -59,6 +59,7 @@ public:
   void on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action, const Glib::RefPtr<Gtk::RadioAction>& operator_action);
   void on_menu_userlevel_Operator(const Glib::RefPtr<Gtk::RadioAction>& action);
 
+  void on_menu_file_export();
   void on_menu_file_print();
 
   void on_menu_Mode_Data();
@@ -100,13 +101,19 @@ public:
   enumModes m_Mode;
   enumModes m_Mode_Previous; // see comments in set_mode_widget().
 
-  static void show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent);
+  static void show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent, Gtk::MessageType message_type = Gtk::MESSAGE_INFO);
 
   //Show the dialog to request the password, and check whether it works.
-  virtual bool connection_request_password_and_attempt();
+  bool connection_request_password_and_attempt();
+
+  //Show the dialog to request the password, and choose an unused database name.
+  bool Frame_Glom::connection_request_password_and_choose_new_database_name();
 
   ///Create the database for new documents, showing the Connection dialog
-  virtual bool create_database(const Glib::ustring& database_name, bool request_password = true);
+  bool create_database(const Glib::ustring& database_name, const Glib::ustring& title, bool request_password = true);
+
+  void export_data_to_string(Glib::ustring& the_string, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause = Glib::ustring());
+  void export_data_to_stream(std::ostream& the_stream, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause = Glib::ustring());
 
 protected:
 
@@ -122,12 +129,16 @@ protected:
   virtual Gtk::Window* get_app_window();
   virtual const Gtk::Window* get_app_window() const;
 
+  void update_records_count();
+
   void alert_no_table();
 
   //Signal handlers:
-  virtual void on_notebook_find_criteria(const Glib::ustring& strWhereClause);
-  virtual void on_notebook_data_record_details_requested(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
-  virtual void on_userlevel_changed(AppState::userlevels userlevel);
+  void on_notebook_find_criteria(const Glib::ustring& strWhereClause);
+  void on_button_quickfind();
+  void on_button_find_all();
+  void on_notebook_data_record_details_requested(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
+  void on_userlevel_changed(AppState::userlevels userlevel);
 
   //Member data:
   Glib::ustring m_strTableName;
@@ -138,6 +149,15 @@ protected:
   Gtk::Label* m_pLabel_Mode;
   Gtk::Label* m_pLabel_userlevel;
 
+  Gtk::HBox* m_pBox_QuickFind; //Only show this when in Find mode.
+  Gtk::Entry* m_pEntry_QuickFind;
+  Gtk::Button* m_pButton_QuickFind;
+
+  Gtk::HBox* m_pBox_RecordsCount; //Only show this when in Data mode.
+  Gtk::Label* m_pLabel_RecordsCount;
+  Gtk::Label* m_pLabel_FoundCount;
+  Gtk::Button* m_pButton_FindAll;
+
   PlaceHolder* m_pBox_Mode; //Contains e.g. design mode notebook.
 
   Box_Tables* m_pBox_Tables;
