@@ -108,6 +108,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   pack_start(m_HBox, Gtk::PACK_SHRINK);
 
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
+  m_FlowTable.signal_field_open_details_requested().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_open_details_requested) );
   show_all();
 
   m_FlowTable.signal_related_record_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_related_record_changed) );
@@ -115,6 +116,8 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
   m_FlowTable.signal_layout_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_layout_changed) );
 
+  m_FlowTable.signal_requested_related_details().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_requested_related_details) );
+    
   m_ignore_signals = false;
 }
 
@@ -137,7 +140,7 @@ bool Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const
   return Box_Data::init_db_details(strTableName); //Calls create_layout(), then fill_from_database()
 }
 
-bool Box_Data_Details::refresh_data_from_database(const Gnome::Gda::Value& primary_key_value)
+bool Box_Data_Details::refresh_data_from_database_with_primary_key(const Gnome::Gda::Value& primary_key_value)
 {
   m_primary_key_value = primary_key_value;
   return fill_from_database();
@@ -145,7 +148,7 @@ bool Box_Data_Details::refresh_data_from_database(const Gnome::Gda::Value& prima
 
 bool Box_Data_Details::refresh_data_from_database_blank()
 {
-  return refresh_data_from_database( Gnome::Gda::Value() );
+  return refresh_data_from_database_with_primary_key( Gnome::Gda::Value() );
 }
 
 void Box_Data_Details::create_layout()
@@ -163,7 +166,7 @@ void Box_Data_Details::create_layout()
     m_FlowTable.set_table(m_strTableName); //This allows portals to get full Relationship information
 
     //This map of layout groups will also contain the field information from the database:
-    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
+    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
 
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
@@ -179,6 +182,10 @@ bool Box_Data_Details::fill_from_database()
 
   Bakery::BusyCursor(*get_app_window());
 
+  const bool primary_key_is_empty = GlomConversions::value_is_empty(m_primary_key_value);
+  if(!primary_key_is_empty)
+    get_document()->set_layout_record_viewed(m_strTableName, m_layout_name, m_primary_key_value);
+
   try
   {
 
@@ -212,9 +219,12 @@ bool Box_Data_Details::fill_from_database()
         const int index_primary_key = fieldsToGet.size() - 1;
 
         const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, m_field_primary_key, m_primary_key_value);
-        Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+        Glib::RefPtr<Gnome::Gda::DataModel> result;
 
-        if(result && result->get_n_rows())
+        if(!primary_key_is_empty)
+          result = Query_execute(query);
+
+        if((result && result->get_n_rows()) || primary_key_is_empty) //either a working result or no result needed.
         {
           const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
           if(pDoc)
@@ -223,11 +233,18 @@ bool Box_Data_Details::fill_from_database()
             //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
 
             const int row_number = 0; //The only row.
-            const int cols_count = result->get_n_columns();
+            int cols_count = 0;
+            if(!primary_key_is_empty)
+              cols_count = result->get_n_columns();
+            else
+              cols_count = fieldsToGet.size();
 
             //Get special possibly-non-visible field values:
-            if(index_primary_key < cols_count)
-              m_primary_key_value = result->get_value_at(index_primary_key, row_number);
+            if(!primary_key_is_empty)
+            {
+              if(index_primary_key < cols_count)
+                m_primary_key_value = result->get_value_at(index_primary_key, row_number);
+            }
 
             //Get field values to show:
             for(int i = 0; i < cols_count; ++i)
@@ -235,7 +252,15 @@ bool Box_Data_Details::fill_from_database()
               sharedptr<LayoutItem_Field> layout_item = fieldsToGet[i];
 
               //Field value:
-              Gnome::Gda::Value value = result->get_value_at(i, row_number);
+              Gnome::Gda::Value value;
+
+              if(!primary_key_is_empty)
+                value = result->get_value_at(i, row_number);
+              else
+              {
+                value = GlomConversions::get_empty_value(layout_item->m_field.get_glom_type());
+              }
+
               m_FlowTable.set_field_value(*layout_item, value);
             }
           }
@@ -254,7 +279,7 @@ bool Box_Data_Details::fill_from_database()
     fill_end();
 
   }
-  catch(std::exception& ex)
+  catch(const std::exception& ex)
   {
     handle_error(ex);
     bResult = false;
@@ -273,7 +298,7 @@ void Box_Data_Details::on_button_new()
       Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name()); //TODO: This should return a Gda::Value
 
       record_new(false /* use entered field data */, primary_key_value);
-      refresh_data_from_database(primary_key_value);
+      refresh_data_from_database_with_primary_key(primary_key_value);
     }
     else
     {
@@ -291,19 +316,22 @@ void Box_Data_Details::on_button_del()
   {
     //Tell user that a primary key is needed to delete a record:
     Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No primary key value.")), true);
-    dialog.set_secondary_text(_("This record can not be deleted because there is no primary key."));
+    dialog.set_secondary_text(_("This record cannot be deleted because there is no primary key."));
     dialog.set_transient_for(*get_app_window());
     dialog.run();
   }
   else
   {
-    bool bTest = record_delete(m_primary_key_value);
-
-    if(bTest)
+    if(confirm_delete_record())
     {
-      //Tell the list that it has been deleted:
-      //It will go to the next (or last) record,
-      signal_record_deleted().emit(m_primary_key_value);
+      bool bTest = record_delete(m_primary_key_value);
+  
+      if(bTest)
+      {
+        //Tell the list that it has been deleted:
+        //It will go to the next (or last) record,
+        signal_record_deleted().emit(m_primary_key_value);
+      }
     }
   }
 }
@@ -347,13 +375,6 @@ Gnome::Gda::Value Box_Data_Details::get_primary_key_value_selected()
   return m_primary_key_value;
 }
 
-/*
-void Box_Data_Details::on_related_user_requested_details(Gnome::Gda::Value key_value, Glib::ustring strTableName)
-{
-  signal_user_requested_related_details().emit(strTableName, key_value);
-}
-*/
-
 void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustring& relationship_name)
 {
   m_FieldsCalculationInProgress.clear();
@@ -439,19 +460,22 @@ Box_Data_Details::type_signal_record_deleted Box_Data_Details::signal_record_del
   return m_signal_record_deleted; 
 }
 
-/*
-Box_Data_Details::type_signal_user_requested_related_details Box_Data_Details::signal_user_requested_related_details()
+Box_Data_Details::type_signal_requested_related_details Box_Data_Details::signal_requested_related_details()
 {
-  return m_signal_user_requested_related_details;
+  return m_signal_requested_related_details;
 }
-*/
 
 void Box_Data_Details::on_flowtable_layout_changed()
 {
   //Get new layout:
   Document_Glom::type_mapLayoutGroupSequence layout_groups;
   m_FlowTable.get_layout_groups(layout_groups);
-  
+
+  //Store it in the document:
+  Document_Glom* document = get_document();
+  if(document)
+    document->set_data_layout_groups(m_layout_name, m_strTableName, layout_groups);
+
   //Build the view again from the new layout:
   create_layout();
 
@@ -459,11 +483,32 @@ void Box_Data_Details::on_flowtable_layout_changed()
   fill_from_database();
 }
 
+void Box_Data_Details::on_flowtable_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
+{
+  if(GlomConversions::value_is_empty(primary_key_value))
+    return; //Ignore empty ID fields.
+
+  signal_requested_related_details().emit(table_name, primary_key_value);
+}
+
 void Box_Data_Details::on_flowtable_related_record_changed(const Glib::ustring& relationship_name)
 {
   recalculate_fields_for_related_records(relationship_name);
 }
 
+void Box_Data_Details::on_flowtable_field_open_details_requested(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
+{
+  if(GlomConversions::value_is_empty(field_value))
+    return; //Ignore empty ID fields.
+    
+  Relationship relationship;
+  const bool related_to_one = get_document()->get_field_used_in_relationship_to_one(m_strTableName, layout_field.get_name(), relationship);
+  if(related_to_one)
+  {
+    signal_requested_related_details().emit(relationship.get_to_table(), field_value);
+  }
+}
+
 void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
@@ -472,7 +517,7 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   const Glib::ustring strFieldName = layout_field.get_name();
 
   Gnome::Gda::Value primary_key_value = get_primary_key_value();
-  if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a primary key value:
+  if(!GlomConversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
   {
     Glib::ustring table_name;
     Field primary_key_field;
@@ -589,6 +634,14 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
         dialog.set_transient_for(*get_app_window());
         dialog.run();
       }
+      else
+      {
+        //Make a new record, and show it:
+        Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name());
+
+        record_new(true /* use entered field data */, primary_key_value);
+        refresh_data_from_database_with_primary_key(primary_key_value);
+      }
     }
     else
     {
@@ -787,7 +840,7 @@ void Box_Data_Details::print_layout()
     //The groups:
     xmlpp::Element* nodeParent = nodeRoot;
 
-    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
+    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
       const LayoutGroup& layout_group = iter->second;
