@@ -22,6 +22,7 @@
 #include "../data_structure/glomconversions.h"
 #include "dialog_layout_list.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
+//#include <../utility_widgets/db_adddel/glom_db_treemodel.h> //For DbTreeModel.
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
@@ -76,7 +77,7 @@ Box_Data_List::~Box_Data_List()
 void Box_Data_List::enable_buttons()
 {
   const Privileges table_privs = get_current_privs(m_strTableName);
-  
+
     //Enable/Disable record creation and deletion:
   bool allow_create = !m_read_only;
   bool allow_delete = !m_read_only;
@@ -85,15 +86,23 @@ void Box_Data_List::enable_buttons()
     allow_create = table_privs.m_create;
     allow_delete = table_privs.m_delete;
   }
-  
+
   m_AddDel.set_allow_add(allow_create);
   m_AddDel.set_allow_delete(allow_delete);
-  
+
   m_AddDel.set_allow_view_details(table_privs.m_view);
 }
 
+void Box_Data_List::refresh_data_from_database_blank()
+{
+  m_AddDel.refresh_from_database_blank();
+  m_AddDel.set_where_clause("");
+}
+
 bool Box_Data_List::fill_from_database()
 {
+  //std::cout << "Box_Data_List::fill_from_database(): where_clause=" << m_strWhereClause << std::endl;
+
   bool result = false;
 
   Bakery::BusyCursor(*get_app_window());
@@ -148,7 +157,7 @@ void Box_Data_List::on_adddel_user_requested_add()
 {
  if(m_FieldsShown.empty())
     return; //Don't try to add a record to a list with no fields.
-    
+
   Gtk::TreeModel::iterator iter = m_AddDel.get_item_placeholder();
   if(iter)
   {
@@ -179,10 +188,14 @@ void Box_Data_List::on_adddel_user_requested_add()
       guint treemodel_column = 0;
       bool test = get_field_column_index(m_FieldsShown[index_field_to_edit]->get_name(), treemodel_column);
       if(test)
+      {
+        std::cout << "on_adddel_user_requested_add(): editing column=" << treemodel_column << std::endl;
         m_AddDel.select_item(iter, treemodel_column, true /* start_editing */);
+      }
     }
     else
     {
+      std::cout << "on_adddel_user_requested_add(): no editable rows." << std::endl;
       //The only keys are non-editable, so just add a row:
       on_adddel_user_added(iter, 0);
       m_AddDel.select_item(iter); //without start_editing.
@@ -222,6 +235,8 @@ void Box_Data_List::on_adddel_user_requested_delete(const Gtk::TreeModel::iterat
 
 void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint col_with_first_value)
 {
+  //std::cout << "Box_Data_List::on_adddel_user_added" << std::endl;
+
   Gnome::Gda::Value primary_key_value;
 
   Field field_primary_key = m_AddDel.get_key_field();
@@ -370,6 +385,7 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
 
       //Update the field in the record (the record with this primary key):
       const Gnome::Gda::Value field_value = m_AddDel.get_value(row, layout_field);
+      std::cout << "Box_Data_List::on_adddel_user_changed(): field_value = " << field_value.to_string() << std::endl;
       //const Field& field = layout_field.m_field;
       //const Glib::ustring strFieldName = layout_field.get_name();
 
@@ -638,9 +654,9 @@ void Box_Data_List::set_entered_field_data(const LayoutItem_Field& field, const
   return m_AddDel.set_value_selected(field, value);
 }
 
-guint Box_Data_List::get_records_count() const
+bool Box_Data_List::get_showing_multiple_records() const
 {
-  return m_AddDel.get_count();
+  return m_AddDel.get_count() > 1;
 }
 
 void Box_Data_List::create_layout()
@@ -682,7 +698,7 @@ void Box_Data_List::create_layout()
         sharedptr<LayoutItem_Field> field = *iter;
         if(m_read_only)
           field->set_editable(false);
-          
+
         m_AddDel.add_column(*field);
       }
 
@@ -738,7 +754,7 @@ bool Box_Data_List::get_field_primary_key(Field& field) const
 
 bool Box_Data_List::get_field_primary_key_index(guint& field_column) const
 {
-  return Box_Data::get_field_primary_key_index(m_FieldsShown, field_column);
+  return Box_Data::get_field_primary_key_index_for_fields(m_FieldsShown, field_column);
 }
 
 void Box_Data_List::print_layout()
@@ -793,6 +809,14 @@ void Box_Data_List::set_primary_key_value_selected(const Gnome::Gda::Value& prim
   }
 }
 
+void Box_Data_List::get_record_counts(gulong& total, gulong& found) const
+{
+  Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
+  Glib::RefPtr<DbTreeModel> refModelDerived = Glib::RefPtr<DbTreeModel>::cast_dynamic(refModel);
+
+  refModelDerived->get_record_counts(total, found);
+}
+
 
 
 
