@@ -30,6 +30,8 @@ public:
   Box_Data_List();
   virtual ~Box_Data_List();
 
+  void refresh_data_from_database_blank();
+
   virtual Gnome::Gda::Value get_primary_key_value(const Gtk::TreeModel::iterator& row);
   virtual Gnome::Gda::Value get_primary_key_value_selected();
   Gnome::Gda::Value get_primary_key_value_first();
@@ -37,7 +39,7 @@ public:
   virtual Gnome::Gda::Value get_entered_field_data(const LayoutItem_Field& field) const;
   virtual void set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value);
 
-  virtual guint get_records_count() const;
+  bool get_showing_multiple_records() const;
 
   void set_read_only(bool read_only = true);
 
@@ -58,6 +60,8 @@ public:
   virtual void on_details_nav_last();
   virtual void on_Details_record_deleted(const Gnome::Gda::Value& primary_key_value);
 
+  void get_record_counts(gulong& total, gulong& found) const;
+
 protected:
   virtual void create_layout(); //override
   virtual bool fill_from_database(); //override.
