@@ -35,15 +35,15 @@ public:
   virtual ~Box_Data_List_Related();
 
   /**
-   * @param Relationship: The relationship used by the parent table to get rows from this table.
+   * @param portal: The full portal details
    */
-  virtual bool init_db_details(const Relationship& relationship);
+  virtual bool init_db_details(const LayoutItem_Portal& portal);
 
   /**
    * @param foreign_key_value: The value that should be found in this table.
    * @param from_table_primary_key_value The primary key of the parent record's table, used to associate new related records.
    */
-  virtual bool refresh_data_from_database(const Gnome::Gda::Value& foreign_key_value);
+  virtual bool refresh_data_from_database_with_foreign_key(const Gnome::Gda::Value& foreign_key_value);
 
   virtual Relationship get_relationship() const;
   virtual Field get_key_field() const;
@@ -60,13 +60,14 @@ protected:
   virtual bool fill_from_database(); //Override.
   virtual type_vecLayoutFields get_fields_to_show() const; //override
 
+  virtual void on_adddel_user_requested_add();
   virtual void on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col);
   virtual void on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint col_with_first_value); //Override.
   virtual void on_record_added(const Gnome::Gda::Value& primary_key_value); //Override. Not a signal handler.
   virtual void on_record_deleted(const Gnome::Gda::Value& primary_key_value); //override.
   virtual void on_dialog_layout_hide(); //override.
 
-  virtual void enable_buttons();
+  virtual void enable_buttons(); //override
 
 protected:
   Gtk::Frame m_Frame;
@@ -74,7 +75,7 @@ protected:
   Gtk::Label m_Label;
   Dialog_Layout_List_Related* m_pDialogLayoutRelated;
 
-  Relationship m_relationship; //The relationship of the parent table to this one.
+  LayoutItem_Portal m_portal;
   Field m_key_field;
   Gnome::Gda::Value m_key_value;
 
