@@ -68,7 +68,7 @@ Dialog_Layout_List::Dialog_Layout_List(BaseObjectType* cobject, const Glib::RefP
     m_model_fields->signal_row_changed().connect( sigc::mem_fun(*this, &Dialog_Layout_List::on_treemodel_row_changed) );
   }
 
- 
+
   refGlade->get_widget("button_field_up", m_button_field_up);
   m_button_field_up->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_List::on_button_field_up) );
 
@@ -120,7 +120,7 @@ void Dialog_Layout_List::set_document(const Glib::ustring& layout, Document_Glom
       guint field_sequence = 1; //0 means no sequence
       for(type_vecLayoutFields::const_iterator iter = table_fields.begin(); iter != table_fields.end(); ++iter)
       {
-        LayoutItem_Field item = *iter;
+        LayoutItem_Field item = *(*iter);
         item.m_sequence = field_sequence;
 
         group.add_item(item, field_sequence);
@@ -146,13 +146,15 @@ void Dialog_Layout_List::set_document(const Glib::ustring& layout, Document_Glom
       for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
       {
         const LayoutItem_Field* item = dynamic_cast<const LayoutItem_Field*>(iter->second); 
+        if(item)
+        {
+          Gtk::TreeModel::iterator iterTree = m_model_fields->append();
+          Gtk::TreeModel::Row row = *iterTree;
 
-        Gtk::TreeModel::iterator iterTree = m_model_fields->append();
-        Gtk::TreeModel::Row row = *iterTree;
-
-        row[m_ColumnsFields.m_col_layout_item] = *item;
-        row[m_ColumnsFields.m_col_sequence] = field_sequence;
-        ++field_sequence;
+          row[m_ColumnsFields.m_col_layout_item] = *item;
+          row[m_ColumnsFields.m_col_sequence] = field_sequence;
+          ++field_sequence;
+        }
       }
     }
 
@@ -313,6 +315,8 @@ void Dialog_Layout_List::on_button_add_field()
   {
     std::cerr << ex.what() << std::endl;
   }
+
+  enable_buttons();
 }
 
 void Dialog_Layout_List::on_button_delete()
@@ -329,6 +333,8 @@ void Dialog_Layout_List::on_button_delete()
       m_modified = true;
     }
   }
+
+  enable_buttons();
 }
 
 
