@@ -34,6 +34,9 @@ public:
   //Create the layout for the database structure, and fill it with data.
   virtual bool init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause = Glib::ustring());
 
+  ///Get the existing where clause, previously supplied to init_db_details().
+  Glib::ustring get_where_clause() const;
+
   ///Show the details for a particular record, without affecting the list view:
   virtual void show_details(const Gnome::Gda::Value& primary_key_value);
 
@@ -42,6 +45,8 @@ public:
   virtual void do_menu_developer_layout(); //override
   virtual void do_menu_file_print(); //override
 
+  void get_record_counts(gulong& total, gulong& found);
+
   enum dataview
   {
     DATA_VIEW_Details,
@@ -55,6 +60,9 @@ public:
 
 protected:
 
+  ///Show the counts of all records and found records:
+  void update_records_count();
+
   //Signal handlers:
   virtual void on_list_user_requested_details(const Gnome::Gda::Value& primary_key_value);
   void on_Details_user_requested_related_details(const Glib::ustring& strTableName, Gnome::Gda::Value primary_key_value);
