@@ -19,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
- 
+
 #ifndef GLOM_SHAREDPTR_H
 #define GLOM_SHAREDPTR_H
 
@@ -91,7 +91,7 @@ protected:
   inline void ref();
   inline void unref();
 
-  void init_db_details();
+  void init();
 
   size_type* m_pRefCount; //Shared between instances, by copying.
   T_obj* m_pobj; //The underlying instance.
@@ -101,7 +101,7 @@ template< typename T_obj>
 sharedptr<T_obj>::sharedptr()
 : m_pRefCount(0), m_pobj(0)
 {
-  init_db_details();
+  init();
 }
 
 template< typename T_obj>
@@ -126,7 +126,7 @@ sharedptr<T_obj>& sharedptr<T_obj>::operator=(const sharedptr<T_obj>& src)
   if(&src != this)
   {
     //Unref any existing stuff.
-    //operator= can never run before a constructor, so these values will be init_db_detailsd already.
+    //operator= can never run before a constructor, so these values will be initd already.
     if(m_pobj) //The if() might not be required.
     {
       unref(); //Could cause a deallocation.
@@ -151,7 +151,7 @@ sharedptr<T_obj>::~sharedptr()
 template< typename T_obj>
 void sharedptr<T_obj>::clear()
 {
-  init_db_details();
+  init();
 }
 
 template< typename T_obj>
@@ -251,15 +251,16 @@ void sharedptr<T_obj>::unref()
 }
 
 template <class T_obj>
-void sharedptr<T_obj>::init_db_details()
+void sharedptr<T_obj>::init()
 {
   //Forget any previous instance:
   if(m_pobj)
   {
     unref();
-    m_pobj = 0;
-    m_pRefCount = 0;
   }
+
+  m_pobj = 0;
+  m_pRefCount = 0;
 }
 
 
