@@ -45,6 +45,9 @@ Glib::ustring build_sql_select_with_where_clause(const Glib::ustring& table_name
 typedef std::list< std::pair<Gnome::Gda::Value, Gnome::Gda::Value> > type_list_values_with_second;
 type_list_values_with_second get_choice_values(const LayoutItem_Field& field);
 
+/** Guess an appropriate identifier name based on a human-readable title
+ */
+Glib::ustring create_name_from_title(const Glib::ustring& title);
 
 void transform_and_open(const xmlpp::Document& xml_document, const Glib::ustring& xsl_file_path);
 Glib::ustring xslt_process(const xmlpp::Document& xml_document, const std::string& filepath_xslt);
